@@ -93,6 +93,7 @@ typedef struct {
 	GtkWidget      *import_preview_box;
 	GtkWidget      *import_reload_button;
 	GtkWidget      *import_delete_button;
+	GtkWidget      *i_commands_table;
 
 	GtkWidget      *progress_info_image;
 	GtkWidget      *progress_info_label;
@@ -105,7 +106,7 @@ typedef struct {
 	/**/
 
 	Camera              *camera;
-	gboolean             camera_setted;
+	gboolean             camera_setted, view_folder;
         GPContext           *context;
 	CameraAbilitiesList *abilities_list;
 	GPPortInfoList      *port_list;
@@ -117,6 +118,7 @@ typedef struct {
 	char                *local_folder;
 
 	GthImporterOp        current_op;
+	gboolean             async_operation;
 	gboolean             interrupted;
 	gboolean             error;
 	float                target;
@@ -128,6 +130,7 @@ typedef struct {
 
 	GList               *categories_list;
 	GList               *delete_list;
+	GList               *saved_images_list;
 
 	/**/
 
@@ -135,6 +138,8 @@ typedef struct {
 	guint                check_id;
 	GMutex              *yes_or_no;
 	gboolean             thread_done; 
+
+	guint                idle_id;
 } DialogData;
 
 
@@ -153,6 +158,11 @@ destroy_cb (GtkWidget  *widget,
 		data->check_id = 0;
 	}
 
+	if (data->idle_id != 0) {
+		g_source_remove (data->idle_id);	
+		data->idle_id = 0;
+	}
+
 	/**/
 
 	g_mutex_lock (data->yes_or_no);
@@ -166,6 +176,11 @@ destroy_cb (GtkWidget  *widget,
 
 	/**/
 
+	if (data->view_folder)
+		window_go_to_directory (data->window, data->local_folder);
+
+	/**/
+
 	g_free (data->progress_info);
 	g_free (data->msg_text);
 	g_free (data->local_folder);
@@ -176,6 +191,7 @@ destroy_cb (GtkWidget  *widget,
 		g_object_unref (data->camera_present_pixbuf);
 	path_list_free (data->categories_list);
 	path_list_free (data->delete_list);
+	path_list_free (data->saved_images_list);
 	gp_camera_unref (data->camera);
 	gp_context_unref (data->context);
 	gp_abilities_list_free (data->abilities_list);
@@ -245,17 +261,33 @@ ctx_progress_update_func (GPContext    *context,
 
 
 static gboolean
-valid_mime_type (const char *type)
+valid_mime_type (const char *name,
+		 const char *type)
 {
-	const char *mime_types[] = { "image",
-				     "video",
-				     "audio" };
-	int i;
+	int         i;
+	const char *name_ext;
 
-	for (i = 0; i < G_N_ELEMENTS (mime_types); i++) {
-		const char *mime_type = mime_types[i];
-		if (strncasecmp (type, mime_type, strlen (mime_type)) == 0)
-			return TRUE;
+	if ((type != NULL) && (strcmp (type, "") != 0)) {
+		const char *mime_types[] = { "image",
+					     "video",
+					     "audio" };
+		for (i = 0; i < G_N_ELEMENTS (mime_types); i++) {
+			const char *mime_type = mime_types[i];
+			if (strncasecmp (type, mime_type, strlen (mime_type)) == 0)
+				return TRUE;
+		}
+	}	
+	
+	name_ext = get_filename_extension (name);
+	if ((name_ext != NULL) && (strcmp (name_ext, "") != 0)) {
+		const char *exts[] = { "JPG", "JPEG", "PNG", "TIF", "TIFF", "GIF",
+				       "AVI", "MPG", "MPEG", 
+				       "AU", "WAV", "OGG", "MP3", "FLAC" };
+		for (i = 0; i < G_N_ELEMENTS (exts); i++) {
+			const char *ext = exts[i];
+			if (strncasecmp (ext, name_ext, strlen (name_ext)) == 0)
+				return TRUE;
+		}
 	}
 
 	return FALSE;
@@ -283,9 +315,14 @@ get_file_list (DialogData *data,
 			CameraFileInfo  info;
 
 			gp_list_get_name (list, i, &name);
-			gp_camera_file_get_info (data->camera, folder, name, &info, data->context);
+			if (gp_camera_file_get_info (data->camera, 
+						     folder, 
+						     name, 
+						     &info, 
+						     NULL) != GP_OK)
+				continue;
 			
-			if (valid_mime_type (info.file.type)) {
+			if (valid_mime_type (info.file.name, info.file.type)) {
 				char *filename = g_build_filename (folder, name, NULL);
 				file_list = g_list_prepend (file_list, filename);
 			}
@@ -452,10 +489,23 @@ update_info (DialogData *data)
 
 			gtk_widget_show (data->progress_info_box);
 		} 
+
+		gdk_flush ();
 	}
 }
 
 
+static void
+main_dialog_set_sensitive (DialogData *data,
+			   gboolean    value)
+{
+	gtk_widget_set_sensitive (data->select_model_button, value);
+	gtk_widget_set_sensitive (data->import_reload_button, value);
+	gtk_widget_set_sensitive (data->import_delete_button, value);
+	gtk_widget_set_sensitive (data->i_commands_table, value);
+}
+
+
  /**/
 
 
@@ -517,9 +567,16 @@ async_operation_step (gpointer callback_data)
 	g_mutex_unlock (aodata->data->yes_or_no);
 
 	if ((aodata->scan == NULL) || interrupted) {
+		g_mutex_lock (aodata->data->yes_or_no);
+		aodata->data->async_operation = FALSE;
+		g_mutex_unlock (aodata->data->yes_or_no);
+
+		main_dialog_set_sensitive (aodata->data, TRUE);
+
 		if (aodata->done_func != NULL)
 			(*aodata->done_func) (aodata, aodata->data);
 		g_free (aodata);
+
 		return FALSE;
 	}
 	
@@ -548,7 +605,10 @@ async_operation_start (AsyncOperationData *aodata)
 	if (aodata->init_func)
 		(*aodata->init_func) (aodata, aodata->data);
 
+	main_dialog_set_sensitive (aodata->data, FALSE);
+
 	g_mutex_lock (aodata->data->yes_or_no);
+	aodata->data->async_operation = TRUE;
 	aodata->data->interrupted = FALSE;
 	g_mutex_unlock (aodata->data->yes_or_no);
 
@@ -571,7 +631,7 @@ static int
 get_default_icon_size (GtkWidget *widget)
 {
         int icon_width, icon_height;
-                                                                                
+
         gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
                                            GTK_ICON_SIZE_DIALOG,
                                            &icon_width, &icon_height);
@@ -705,7 +765,8 @@ load_images_preview__step (AsyncOperationData *aodata,
 		unlink (tmp_filename);
 	}
 	g_free (tmp_filename);
-	
+	g_free (camera_folder);
+
 	gp_file_unref (file);
 }
 
@@ -750,11 +811,13 @@ load_images_preview (DialogData *data)
 					  GTK_ICON_SIZE_BUTTON);
 		gtk_label_set_text (GTK_LABEL (data->progress_info_label), _("No images found"));
 		gtk_widget_show (data->progress_info_box);
+		gtk_window_set_resizable (GTK_WINDOW (data->dialog), FALSE);
 		return;
 
 	} else {
 		gtk_widget_show (data->import_preview_box);
 		gtk_widget_hide (data->progress_info_box);
+		gtk_window_set_resizable (GTK_WINDOW (data->dialog), TRUE);
 	}
 
 	aodata = async_operation_new (file_list,
@@ -779,6 +842,7 @@ set_camera_model (DialogData *data,
 		gtk_widget_hide (data->import_preview_box);
 		gtk_label_set_text (GTK_LABEL (data->camera_model_label), _("No camera detected"));
 		gtk_image_set_from_pixbuf (GTK_IMAGE (data->progress_camera_image), data->no_camera_pixbuf);
+		gtk_window_set_resizable (GTK_WINDOW (data->dialog), FALSE);
 		return;
 	}
 
@@ -802,6 +866,9 @@ set_camera_model (DialogData *data,
 	} 
 
 	if (r >= 0) {
+		eel_gconf_set_string (PREF_PHOTO_IMPORT_MODEL, model);
+		eel_gconf_set_string (PREF_PHOTO_IMPORT_PORT, port);
+
 		_gtk_label_set_locale_text (GTK_LABEL (data->camera_model_label), model);
 		gtk_image_set_from_pixbuf (GTK_IMAGE (data->progress_camera_image), data->camera_present_pixbuf);
 		load_images_preview (data);
@@ -828,7 +895,7 @@ autodetect_camera (DialogData *data)
 	const char *model = NULL, *port = NULL;
 
 	data->current_op = GTH_IMPORTER_OP_AUTO_DETECT;
-
+		
 	gp_list_new (&list);
 
 	gp_abilities_list_detect (data->abilities_list,
@@ -845,7 +912,7 @@ autodetect_camera (DialogData *data)
 		model = NULL;
 		port = NULL;
 	}
-
+		
 	set_camera_model (data, model, port);
 
 	gp_list_free (list);
@@ -995,7 +1062,7 @@ get_folder_name (DialogData *data)
 
 		time (&now);
 		tm = localtime (&now);
-		strftime (time_txt, 50, "%Y-%m-%d--%H:%M:%S", tm);
+		strftime (time_txt, 50, "%Y-%m-%d--%H.%M.%S", tm);
 	
 		film_name = g_strdup (time_txt);
 	} else
@@ -1125,18 +1192,23 @@ save_image (DialogData *data,
 			    GP_FILE_TYPE_NORMAL,
 			    file, 
 			    data->context);
-	g_free (camera_folder);
 
 	local_path = get_file_name (data, camera_path, local_folder, n);
 
 	if (gp_file_save (file, local_path) >= 0) {
 		if (data->delete_from_camera) 
-			data->delete_list = g_list_prepend (data->delete_list, 
-							    g_strdup (camera_path));
+			data->delete_list = g_list_prepend (data->delete_list, g_strdup (camera_path));
+		data->saved_images_list = g_list_prepend (data->saved_images_list, g_strdup (camera_path));
 		add_categories_to_image (data, local_path);
+	} else {
+		g_mutex_lock (data->yes_or_no);
+		data->error = TRUE;
+		data->interrupted = TRUE;
+		g_mutex_unlock (data->yes_or_no);
 	}
 	
 	g_free (local_path);
+	g_free (camera_folder);
 	gp_file_unref (file);
 }
 
@@ -1190,8 +1262,8 @@ delete_images__done (AsyncOperationData *aodata,
 
 	if (interrupted)
 		return;
-	
-	window_go_to_directory (data->window, data->local_folder);
+
+	data->view_folder = TRUE;
 
 	if (ImportPhotos) {
 		ImportPhotos = FALSE;
@@ -1206,7 +1278,17 @@ static void
 save_images__init (AsyncOperationData *aodata, 
 		   DialogData         *data)
 {
+	all_windows_remove_monitor ();
+
 	data->image_n = 1;
+	if (data->delete_list != NULL) {
+		path_list_free (data->delete_list);
+		data->delete_list = NULL;
+	}
+	if (data->saved_images_list != NULL) {
+		path_list_free (data->saved_images_list);
+		data->saved_images_list = NULL;
+	}
 }
 
 
@@ -1219,18 +1301,42 @@ save_images__step (AsyncOperationData *aodata,
 }
 
 
+static gboolean
+notify_cb (gpointer cb_data)
+{
+	DialogData *data = cb_data;
+
+	g_source_remove (data->idle_id);
+	data->idle_id = 0;
+
+	if (data->saved_images_list != NULL) {
+		all_windows_notify_files_created (data->saved_images_list);
+		path_list_free (data->saved_images_list);
+		data->saved_images_list = NULL;
+	}
+	
+	all_windows_add_monitor ();
+
+	return FALSE;
+}
+
+
 static void 
 save_images__done (AsyncOperationData *aodata, 
 		   DialogData         *data)
 {
 	gboolean            interrupted;
+	gboolean            error;
 	AsyncOperationData *new_aodata;
 
 	g_mutex_lock (data->yes_or_no);
 	interrupted = data->interrupted;
+	error = data->error;
 	g_mutex_unlock (data->yes_or_no);
 
-	if (interrupted)
+	data->idle_id = g_idle_add (notify_cb, data);
+
+	if (interrupted || error)
 		return;
 
 	new_aodata = async_operation_new (data->delete_list,
@@ -1242,6 +1348,25 @@ save_images__done (AsyncOperationData *aodata,
 }
 
 
+static void
+cancel_clicked_cb (GtkButton  *button,
+		   DialogData *data)
+{
+	gboolean async_operation;
+
+	g_mutex_lock (data->yes_or_no);
+	async_operation = data->async_operation;
+	g_mutex_unlock (data->yes_or_no);
+
+	if (async_operation) {
+		g_mutex_lock (data->yes_or_no);
+		data->interrupted = TRUE;
+		g_mutex_unlock (data->yes_or_no);
+	} else
+		gtk_widget_destroy (data->dialog);
+}
+
+
 static void
 ok_clicked_cb (GtkButton  *button,
 	       DialogData *data)
@@ -1250,6 +1375,7 @@ ok_clicked_cb (GtkButton  *button,
 	GList              *sel_list;
 	gboolean            error;
 	AsyncOperationData *aodata;
+	GnomeVFSFileSize    total_size = 0;
 
 	if (!data->camera_setted) {
 		display_error_dialog (data,
@@ -1259,6 +1385,7 @@ ok_clicked_cb (GtkButton  *button,
 	}
 
 	/**/
+
 	g_free (data->local_folder);
 
 	data->local_folder = get_folder_name (data);
@@ -1273,9 +1400,21 @@ ok_clicked_cb (GtkButton  *button,
 
 	/**/
 
+	g_mutex_lock (data->yes_or_no);
+	error = data->error;
+	g_mutex_unlock (data->yes_or_no);
+
+	if (error) {
+		update_info (data);
+		return;
+	}
+
 	sel_list = gth_image_list_get_selection (GTH_IMAGE_LIST (data->image_list));
-	if (sel_list == NULL)
+	if (sel_list == NULL) {
 		sel_list = gth_image_list_get_list (GTH_IMAGE_LIST (data->image_list));
+		g_list_foreach (sel_list, (GFunc) file_data_ref, NULL);
+	}
+
 	if (sel_list != NULL) {
 		for (scan = sel_list; scan; scan = scan->next) {
 			FileData   *fdata = scan->data;
@@ -1287,33 +1426,71 @@ ok_clicked_cb (GtkButton  *button,
 		file_data_list_free (sel_list);
 	} 
 
-	g_mutex_lock (data->yes_or_no);
-	error = data->error;
-	g_mutex_unlock (data->yes_or_no);
-
-	if (error) {
-		update_info (data);
-		return;
-	}
-	
 	if (file_list == NULL) {
 		display_error_dialog (data,
 				      _("Could not import photos"), 
 				      _("No images found"));
 		return;
 	}
-	
+
 	if (! ensure_dir_exists (data->local_folder, 0755)) {
+		char *utf8_path;
 		char *msg;
-		
+		utf8_path = g_filename_to_utf8 (data->local_folder, -1, NULL, NULL, NULL);
 		msg = g_strdup_printf (_("Could not create the folder \"%s\": %s"),
-				       data->local_folder,
-				       errno_to_string());
+				       utf8_path,
+				       errno_to_string ());
 		display_error_dialog (data, _("Could not import photos"), msg);
+
+		g_free (utf8_path);
 		g_free (msg);
-		
 		g_free (data->local_folder);
 		data->local_folder = NULL;
+		path_list_free (file_list);
+		return; 
+	}
+
+	if (! check_permissions (data->local_folder, R_OK | W_OK | X_OK)) {
+		char *utf8_path;
+		char *msg;
+		utf8_path = g_filename_to_utf8 (data->local_folder, -1, NULL, NULL, NULL);
+		msg = g_strdup_printf (_("You don't have the right permissions to create images in the folder \"%s\""), utf8_path);
+
+		display_error_dialog (data, _("Could not import photos"), msg);
+				       
+		g_free (utf8_path);
+		g_free (msg);
+		g_free (data->local_folder);
+		data->local_folder = NULL;
+		path_list_free (file_list);
+		return;
+	} 
+
+	for (scan = file_list; scan; scan = scan->next) {
+		const char     *camera_path = scan->data;
+		CameraFileInfo  info;
+		char           *camera_folder;
+		const char     *camera_filename;
+
+		camera_folder = remove_level_from_path (camera_path);
+		camera_filename = file_name_from_path (camera_path);
+		if (gp_camera_file_get_info (data->camera, 
+					     camera_folder, 
+					     camera_filename,
+					     &info,
+					     NULL) == GP_OK)
+			total_size += (GnomeVFSFileSize) info.file.size;
+		g_free (camera_folder);
+	}
+
+	if (get_dest_free_space (data->local_folder) < total_size) {
+		display_error_dialog (data, 
+				      _("Could not import photos"), 
+				      "Not enough free space left on disk"); /* FIXME: mark for translation. */
+
+		g_free (data->local_folder);
+		data->local_folder = NULL;
+		path_list_free (file_list);
 		return; 
 	}
 
@@ -1403,7 +1580,7 @@ import_delete_cb (GtkButton  *button,
 		camera_filename = file_name_from_path (camera_path);
 		
 		gp_camera_file_delete (data->camera, camera_folder, camera_filename, data->context);
-		/* FIXME */
+		g_free (camera_folder);
 	}
 
 	path_list_free (delete_list);
@@ -1466,8 +1643,20 @@ check_thread (gpointer cb_data)
 		task_terminated (data);
 
 		switch (data->current_op) {
-		case GTH_IMPORTER_OP_LIST_ABILITIES:
-			autodetect_camera (data);
+		case GTH_IMPORTER_OP_LIST_ABILITIES: 
+			if (! autodetect_camera (data)) {
+				char *camera_model;
+				char *camera_port;
+				
+				camera_model = eel_gconf_get_string (PREF_PHOTO_IMPORT_MODEL, NULL);
+				camera_port = eel_gconf_get_string (PREF_PHOTO_IMPORT_PORT, NULL);
+
+				if ((camera_model != NULL) && (camera_port != NULL))
+					set_camera_model (data, camera_model, camera_port);
+
+				g_free (camera_model);
+				g_free (camera_port);
+			}
 			break;
 		default:
 			break;
@@ -1505,6 +1694,7 @@ dlg_photo_importer (GThumbWindow *window)
 	GdkPixbuf    *mute_pixbuf;
 	char         *default_path;
 	char         *default_film_name;
+	GValue        value = {0, };
 
 	data = g_new0 (DialogData, 1);
 	data->window = window;
@@ -1537,6 +1727,7 @@ dlg_photo_importer (GThumbWindow *window)
 
 	data->yes_or_no = g_mutex_new ();
 	data->check_id = 0;
+	data->idle_id = 0;
 	data->msg_text  = NULL;
 
 	/* Get the widgets. */
@@ -1561,6 +1752,7 @@ dlg_photo_importer (GThumbWindow *window)
 	data->import_preview_box = glade_xml_get_widget (data->gui, "import_preview_box");
 	data->import_reload_button = glade_xml_get_widget (data->gui, "import_reload_button");
 	data->import_delete_button = glade_xml_get_widget (data->gui, "import_delete_button");
+	data->i_commands_table = glade_xml_get_widget (data->gui, "i_commands_table");
 	btn_ok = glade_xml_get_widget (data->gui, "import_okbutton");
 	btn_cancel = glade_xml_get_widget (data->gui, "import_cancelbutton");
 
@@ -1570,9 +1762,18 @@ dlg_photo_importer (GThumbWindow *window)
 	gtk_container_add (GTK_CONTAINER (data->import_preview_scrolledwindow), data->image_list);
 
 	gtk_widget_hide (data->import_preview_box);
+	gtk_window_set_resizable (GTK_WINDOW (data->dialog), FALSE);
 
 	/* Set widgets data. */
 
+	/* Make use of the new filechooser */
+
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, TRUE);
+	g_object_set_property (G_OBJECT (data->destination_fileentry), "use_filechooser", &value);
+
+	/**/
+
 	data->camera_present_pixbuf = gdk_pixbuf_new_from_file (GTHUMB_GLADEDIR "/" CAMERA_FILE, NULL);
 	mute_pixbuf = gdk_pixbuf_new_from_file (GTHUMB_GLADEDIR "/" MUTE_FILE, NULL);
 
@@ -1619,10 +1820,10 @@ dlg_photo_importer (GThumbWindow *window)
 			  "clicked",
 			  G_CALLBACK (ok_clicked_cb),
 			  data);
-	g_signal_connect_swapped (G_OBJECT (btn_cancel), 
-				  "clicked",
-				  G_CALLBACK (gtk_widget_destroy),
-				  G_OBJECT (data->dialog));
+	g_signal_connect (G_OBJECT (btn_cancel), 
+			  "clicked",
+			  G_CALLBACK (cancel_clicked_cb),
+			  data);
 	g_signal_connect (G_OBJECT (data->select_model_button), 
 			  "clicked",
 			  G_CALLBACK (dlg_select_camera_model_cb),
